﻿using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using SharQL.Ast.Nodes;
using SharQL.Algebra.AstUtils.Visitor;
using SharQL.Algebra.Tests.Utils;

namespace SharQL.Algebra.AstUtils.Tests
{
	/// <summary>
	/// Summary description for BlankSubjectReificationTest
	/// </summary>
	[TestClass]
	public class BlankNodeReificationTest
	{
		public BlankNodeReificationTest()
		{

		}

		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		//
		// You can use the following additional attributes as you write your tests:
		//
		// Use ClassInitialize to run code before running the first test in the class
		// [ClassInitialize()]
		// public static void MyClassInitialize(TestContext testContext) { }
		//
		// Use ClassCleanup to run code after all tests in a class have run
		// [ClassCleanup()]
		// public static void MyClassCleanup() { }
		//
		// Use TestInitialize to run code before running each test 
		// [TestInitialize()]
		// public void MyTestInitialize() { }
		//
		// Use TestCleanup to run code after each test has run
		// [TestCleanup()]
		// public void MyTestCleanup() { }
		//
		#endregion

		[TestMethod]
		public void TestEmptyBrackets()
		{
			QueryNode root = createQueryWithEmptyBrackets();
			reificateBlankSubjects(ref root);
			verifyReificationOfLonelyBrackets(root);
		}

		[TestMethod]
		public void TestLonelyBrackets()
		{
			QueryNode root = createQueryWithLonelyBrackets();
			reificateBlankSubjects(ref root);
			verifyReificationOfLonelyBrackets(root);
		}

		[TestMethod]
		public void TestBracketsAsSubject()
		{
			QueryNode root = createQueryWithBracketsAsSubject();
			reificateBlankSubjects(ref root);
			verifyReificationOfBracketsAsSubject(root);
		}

		[TestMethod]
		public void TestBracketsAsObject()
		{
			QueryNode root = createQueryWithBracketsAsObject();
			reificateBlankSubjects(ref root);
			verifyReificationOfBracketsAsObject(root);
		}

		[TestMethod]
		public void TestCompositeBrackets()
		{
			QueryNode root = createQueryWithCompositeBrackets();
			reificateBlankSubjects(ref root);
			verifyReificationOfCompositeBrackets(root);
		}

		[TestMethod]
		public void TestNestedBrackets()
		{
			QueryNode root = createQueryWithNestedBrackets();
			reificateBlankSubjects(ref root);
			verifyReificationOfNestedBrackets(root);
		}

		/// <summary>
		/// <code>
		/// SELECT *
		/// {
		/// [] <http://example.org/prop> "v" .
		/// } 
		/// </code>
		/// </summary>
		private QueryNode createQueryWithEmptyBrackets()
		{
			return new QueryNode(
				new PrologueNode(null, null),
				new SelectQueryNode(
					new StarVarListNode() { IsStar = true },
					new WhereClauseNode(
						new GroupGraphPatternNode(
							new TriplesBlockNode(
								new TriplesSameSubjectNode(
									new BlankNode(),
                                    null,
									new PropertyListNode(
										new IriRefNode() { Value = "http://example.org/prop" },
										new ObjectListNode(
											new RdfLiteralNode(
												new StringLiteralNode() { Value = "v" },
												null
												),
											null
										),
										null
									)
								),
								null
							),
							null
						)
					)
				)
			);
		}

		/// <summary>
		/// <code>
		/// SELECT *
		/// {
		/// [ <http://example.org/prop> "v" ] .
		/// } 
		/// </code>
		/// </summary>
		private QueryNode createQueryWithLonelyBrackets()
		{
			return new QueryNode(
				new PrologueNode(null, null),
				new SelectQueryNode(
					new StarVarListNode() { IsStar = true },
					new WhereClauseNode(
						new GroupGraphPatternNode(
							new TriplesBlockNode(
								new TriplesSameSubjectNode(
									new BlankNodePropertyListNode(
										new PropertyListNode(
											new IriRefNode() { Value = "http://example.org/prop" },
											new ObjectListNode(
												new RdfLiteralNode(
													new StringLiteralNode() { Value = "v" },
													null
													),
												null
											),
											null
										)
									),
									null,
									null
								),
								null
							),
							null
						)
					)
				)
			);
		}

		/// <summary>
		/// <code>
		/// SELECT *
		/// {
		/// [ <http://example.org/prop> "v" ] <http://example.org/prop> "w" .
		/// } 
		/// </code>
		/// </summary>
		private QueryNode createQueryWithBracketsAsSubject()
		{
			return new QueryNode(
				new PrologueNode(null, null),
				new SelectQueryNode(
					new StarVarListNode() { IsStar = true },
					new WhereClauseNode(
						new GroupGraphPatternNode(
							new TriplesBlockNode(
								new TriplesSameSubjectNode(
									new BlankNodePropertyListNode(
										new PropertyListNode(
											new IriRefNode() { Value = "http://example.org/prop" },
											new ObjectListNode(
												new RdfLiteralNode(
													new StringLiteralNode() { Value = "v" },
													null
													),
												null
											),
											null
										)
									),
									null,
									new PropertyListNode(
										new IriRefNode() { Value = "http://example.org/prop" },
										new ObjectListNode(
											new RdfLiteralNode(
												new StringLiteralNode() { Value = "w" },
												null
												),
											null
										),
										null
									)
								),
								null
							),
							null
						)
					)
				)
			);			
		}

		/// <summary>
		/// <code>
		/// SELECT *
		/// {
		/// "w" <http://example.org/prop> [ <http://example.org/prop> "v" ] .
		/// } 
		/// </code>
		/// </summary>
		private QueryNode createQueryWithBracketsAsObject()
		{
			return new QueryNode(
				new PrologueNode(null, null),
				new SelectQueryNode(
					new StarVarListNode() { IsStar = true },
					new WhereClauseNode(
						new GroupGraphPatternNode(
							new TriplesBlockNode(
								new TriplesSameSubjectNode(
									new RdfLiteralNode(
										new StringLiteralNode() { Value = "w" },
										null
									),
									null,
									new PropertyListNode(
										new IriRefNode() { Value = "http://example.org/prop" },
										new ObjectListNode(
											new BlankNodePropertyListNode(
												new PropertyListNode(
													new IriRefNode() { Value = "http://example.org/prop" },
													new ObjectListNode(
														new RdfLiteralNode(
															new StringLiteralNode() { Value = "v" },
															null
														),
														null
													),
													null
												)
											),
											null
										),
										null
									)
								),
								null
							),
							null
						)
					)
				)
			);
		}

		/// <summary>
		/// <code>
		/// SELECT *
		/// {
		/// [ <http://example.org/prop> "w" ; <http://example.org/prop> "v" ] .
		/// }
		/// </code>
		/// </summary>
		private QueryNode createQueryWithCompositeBrackets()
		{
			return new QueryNode(
				new PrologueNode(null, null),
				new SelectQueryNode(
					new StarVarListNode() { IsStar = true },
					new WhereClauseNode(
						new GroupGraphPatternNode(
							new TriplesBlockNode(
								new TriplesSameSubjectNode(
									new BlankNodePropertyListNode(
										new PropertyListNode(
											new IriRefNode() { Value = "http://example.org/prop" },
											new ObjectListNode(
												new RdfLiteralNode(
													new StringLiteralNode() { Value = "w" },
													null
												),
												null
											),
											new PropertyListNode(
												new IriRefNode() { Value = "http://example.org/prop" },
												new ObjectListNode(
													new RdfLiteralNode(
														new StringLiteralNode() { Value = "v" },
														null
													),
													null
												),
												null
											)
										)
									),
									null,
									null
								),
								null
							),
							null
						)
					)
				)
			);
		}

		/// <summary>
		/// <code>
		/// SELECT *
		/// {
		/// [?a [?b ?c ; ?d ?e , ?f]] .
		/// }
		/// </code>
		/// </summary>
		private QueryNode createQueryWithNestedBrackets()
		{
			return new QueryNode(
				new PrologueNode(null, null),
				new SelectQueryNode(
					new StarVarListNode() { IsStar = true },
					new WhereClauseNode(
						new GroupGraphPatternNode(
							new TriplesBlockNode(
								new TriplesSameSubjectNode(
									new BlankNodePropertyListNode(
										new PropertyListNode(
											new VarNode("a"),
											new ObjectListNode(
												new BlankNodePropertyListNode(
													new PropertyListNode(
														new VarNode("b"),
														new ObjectListNode(
															new VarNode("c"),
															null
														),
														new PropertyListNode(
															new VarNode("d"),
															new ObjectListNode(
																new VarNode("e"),
																new ObjectListNode(
																	new VarNode("f"),
																	null
																)
															),
															null
														)
													)
												),
												null
											),
											null
										)
									),
									null,
									null
								),
								null
							),
							null
						)
					)
				)
			);
		}

		private void reificateBlankSubjects(ref QueryNode root)
		{
            BlankNodeReificationVisitor visitor = new BlankNodeReificationVisitor();
            root.Accept(visitor);
        }

		private void verifyReificationOfLonelyBrackets(QueryNode root)
		{
			TriplesBlockNode[] triples = TripleUtils.FetchSubsequentTriples(root.Children[1].Children[1].Children[0].Children[0] as TriplesBlockNode);
			List<TriplesBlockNode> triplesList = new List<TriplesBlockNode>(triples);

			Assert.AreEqual(1, triplesList.Count);

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new IriRefNode() { Value = "http://example.org/prop" },
				new RdfLiteralNode(new StringLiteralNode() { Value = "v" }, null)));

			Assert.AreEqual(0, triplesList.Count);
		}

		private void verifyReificationOfBracketsAsSubject(QueryNode root)
		{
			TriplesBlockNode[] triples = TripleUtils.FetchSubsequentTriples(root.Children[1].Children[1].Children[0].Children[0] as TriplesBlockNode);
			List<TriplesBlockNode> triplesList = new List<TriplesBlockNode>(triples);

			Assert.AreEqual(2, triplesList.Count);

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new IriRefNode() { Value = "http://example.org/prop" },
				new RdfLiteralNode(new StringLiteralNode() { Value = "v" }, null)));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new IriRefNode() { Value = "http://example.org/prop" },
				new RdfLiteralNode(new StringLiteralNode() { Value = "w" }, null)));

			Assert.AreEqual(0, triplesList.Count);
			Assert.AreEqual((TripleUtils.GetSubject(triples[0]) as BlankNode).Value,
				(TripleUtils.GetSubject(triples[1]) as BlankNode).Value);
		}

		private void verifyReificationOfBracketsAsObject(QueryNode root)
		{
			TriplesBlockNode[] triples = TripleUtils.FetchSubsequentTriples(root.Children[1].Children[1].Children[0].Children[0] as TriplesBlockNode);
			List<TriplesBlockNode> triplesList = new List<TriplesBlockNode>(triples);

			Assert.AreEqual(2, triplesList.Count);

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new RdfLiteralNode(new StringLiteralNode() { Value = "w" }, null),
				new IriRefNode() { Value = "http://example.org/prop" },
				new BlankNode()));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new IriRefNode() { Value = "http://example.org/prop" },
				new RdfLiteralNode(new StringLiteralNode() { Value = "v" }, null)));

			Assert.AreEqual(0, triplesList.Count);
			Assert.AreEqual((TripleUtils.GetObject(triples[0]) as BlankNode).Value,
				(TripleUtils.GetSubject(triples[1]) as BlankNode).Value);
		}

		private void verifyReificationOfCompositeBrackets(QueryNode root)
		{
			TriplesBlockNode[] triples = TripleUtils.FetchSubsequentTriples(root.Children[1].Children[1].Children[0].Children[0] as TriplesBlockNode);
			List<TriplesBlockNode> triplesList = new List<TriplesBlockNode>(triples);

			Assert.AreEqual(2, triplesList.Count);

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new IriRefNode() { Value = "http://example.org/prop" },
				new RdfLiteralNode(new StringLiteralNode() { Value = "w" }, null)));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new IriRefNode() { Value = "http://example.org/prop" },
				new RdfLiteralNode(new StringLiteralNode() { Value = "v" }, null)));

			Assert.AreEqual(0, triplesList.Count);
			Assert.AreEqual((TripleUtils.GetSubject(triples[0]) as BlankNode).Value,
				(TripleUtils.GetSubject(triples[1]) as BlankNode).Value);
		}

		private void verifyReificationOfNestedBrackets(QueryNode root)
		{
			TriplesBlockNode[] triples = TripleUtils.FetchSubsequentTriples(root.Children[1].Children[1].Children[0].Children[0] as TriplesBlockNode);
			List<TriplesBlockNode> triplesList = new List<TriplesBlockNode>(triples);

			Assert.AreEqual(4, triplesList.Count);

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new VarNode("a"),
				new BlankNode()));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new VarNode("b"),
				new VarNode("c")));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new VarNode("d"),
				new VarNode("e")));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new VarNode("d"),
				new VarNode("f")));

			Assert.AreEqual(0, triplesList.Count);
		}
	}
}
