﻿using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using SharQL.Ast.Nodes;
using SharQL.Algebra.AstUtils.Visitor;
using SharQL.Algebra.Tests.Utils;

namespace SharQL.Algebra.AstUtils.Tests
{
	/// <summary>
	/// Summary description for BracketsInListTest
	/// </summary>
	[TestClass]
	public class BracketsInListTest
	{
		public BracketsInListTest()
		{
		}

		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		//
		// You can use the following additional attributes as you write your tests:
		//
		// Use ClassInitialize to run code before running the first test in the class
		// [ClassInitialize()]
		// public static void MyClassInitialize(TestContext testContext) { }
		//
		// Use ClassCleanup to run code after all tests in a class have run
		// [ClassCleanup()]
		// public static void MyClassCleanup() { }
		//
		// Use TestInitialize to run code before running each test 
		// [TestInitialize()]
		// public void MyTestInitialize() { }
		//
		// Use TestCleanup to run code after each test has run
		// [TestCleanup()]
		// public void MyTestCleanup() { }
		//
		#endregion

		[TestMethod]
		public void TestBracketsInList()
		{
			QueryNode root = createQueryWithBracketsInList();
			reificateBracketsAndLists(root);
			verifyBracketsInList(root);
		}

		[TestMethod]
		public void TestListInBrackets()
		{
			QueryNode root = createQueryWithListInBrackets();
			reificateBracketsAndLists(root);
			verifyListInBrackets(root);
		}

		/// <summary>
		/// <code>
		/// SELECT *
		/// {
		/// (?a [?b ?c]) .
		/// }
		/// </code>
		/// </summary>
		private QueryNode createQueryWithBracketsInList()
		{
			return new QueryNode(
				new PrologueNode(null, null),
				new SelectQueryNode(
					new StarVarListNode() { IsStar = true },
					new WhereClauseNode(
						new GroupGraphPatternNode(
							new TriplesBlockNode(
								new TriplesSameSubjectNode(
									new CollectionNode(
										new GraphNodeListNode(
											new VarNode("a"),
											new GraphNodeListNode(
												new BlankNodePropertyListNode(
													new PropertyListNode(
														new VarNode("b"),
														new ObjectListNode(
															new VarNode("c"),
															null
														),
														null
													)
												),
												null
											)
										)
									),
									null,
									null
								),
								null
							),
							null
						)
					)
				)
			);
		}

		/// <summary>
		/// <code>
		/// SELECT *
		/// {
		/// [?a (?b ?c)] .
		/// }
		/// </code>
		/// </summary>
		private QueryNode createQueryWithListInBrackets()
		{
			return new QueryNode(
				new PrologueNode(null, null),
				new SelectQueryNode(
					new StarVarListNode() { IsStar = true },
					new WhereClauseNode(
						new GroupGraphPatternNode(
							new TriplesBlockNode(
								new TriplesSameSubjectNode(
									new BlankNodePropertyListNode(
										new PropertyListNode(
											new VarNode("a"),
											new ObjectListNode(
												new CollectionNode(
													new GraphNodeListNode(
														new VarNode("b"),
														new GraphNodeListNode(
															new VarNode("c"),
															null
														)
													)
												),
												null
											),
											null
										)
									),
									null,
									null
								),
								null
							),
							null
						)
					)
				)
			);
		}

		private void reificateBracketsAndLists(QueryNode root)
		{
            root.Accept(new ListReificationVisitor());
            root.Accept(new BlankNodeReificationVisitor());
		}

		private void verifyBracketsInList(QueryNode root)
		{
			TriplesBlockNode[] triples = TripleUtils.FetchSubsequentTriples(root.Children[1].Children[1].Children[0].Children[0] as TriplesBlockNode);
			List<TriplesBlockNode> triplesList = new List<TriplesBlockNode>(triples);

			Assert.AreEqual(5, triplesList.Count);

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new IriRefNode() { Value = "http://www.w3.org/1999/02/22-rdf-syntax-ns#first" },
				new VarNode("a")));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new IriRefNode() { Value = "http://www.w3.org/1999/02/22-rdf-syntax-ns#rest" },
				new BlankNode()));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new IriRefNode() { Value = "http://www.w3.org/1999/02/22-rdf-syntax-ns#first" },
				new BlankNode()));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new IriRefNode() { Value = "http://www.w3.org/1999/02/22-rdf-syntax-ns#rest" },
				new IriRefNode() { Value = "http://www.w3.org/1999/02/22-rdf-syntax-ns#nil" }));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new VarNode("b"),
				new VarNode("c")));

			Assert.AreEqual(0, triplesList.Count);

			// TODO: Check blank nodes
		}

		private void verifyListInBrackets(QueryNode root)
		{
			TriplesBlockNode[] triples = TripleUtils.FetchSubsequentTriples(root.Children[1].Children[1].Children[0].Children[0] as TriplesBlockNode);
			List<TriplesBlockNode> triplesList = new List<TriplesBlockNode>(triples);

			Assert.AreEqual(5, triplesList.Count);

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new VarNode("a"),
				new BlankNode()));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new IriRefNode() { Value = "http://www.w3.org/1999/02/22-rdf-syntax-ns#first" },
				new VarNode("b")));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new IriRefNode() { Value = "http://www.w3.org/1999/02/22-rdf-syntax-ns#rest" },
				new BlankNode()));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new IriRefNode() { Value = "http://www.w3.org/1999/02/22-rdf-syntax-ns#first" },
				new VarNode("c")));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new BlankNode(),
				new IriRefNode() { Value = "http://www.w3.org/1999/02/22-rdf-syntax-ns#rest" },
				new IriRefNode() { Value = "http://www.w3.org/1999/02/22-rdf-syntax-ns#nil" }));

			Assert.AreEqual(0, triplesList.Count);

			// TODO: Check blank nodes
		}
	}
}
