﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using SharQL.Ast;
using SharQL.Ast.Nodes;
using SharQL.Algebra.AstUtils.Visitor;

namespace SharQL.Algebra.AstUtils.Tests
{
	/// <summary>
	///This is a test class for LiteralExplicatorVisitor and is intended
	///to contain all LiteralExplicatorVisitor Unit Tests
	///</summary>
	[TestClass]
	public class LiteralExplicatorVisitorTest
	{
		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion

		private void explicateLiterals(ref INode node)
		{
			LiteralExplicatorVisitor visitor = new LiteralExplicatorVisitor();
			INode parentNode = new VarNode("dummy");
			parentNode.Children.Insert(0, node);
			node.Parent = parentNode;
			parentNode.Accept(visitor);
			node = parentNode.Children[0];
		}

		[TestMethod]
		public void TestLiteralExplication()
		{
			INode node = new NumericLiteralNode()
			{
				IntValue = 123,
				Type = NumericLiteralNode.LiteralType.Int
			};
			explicateLiterals(ref node);
			Assert.IsInstanceOfType(node, typeof(RdfLiteralNode));
			Assert.AreEqual("123", ((node as RdfLiteralNode).String as StringLiteralNode).Value);
			Assert.AreEqual("http://www.w3.org/2001/XMLSchema#integer", ((node as RdfLiteralNode).IriRef as IriRefNode).Value);

			node = new NumericLiteralNode()
			{
				DoubleValue = 123e10,
				Type = NumericLiteralNode.LiteralType.Double
			};
			explicateLiterals(ref node);
			Assert.IsInstanceOfType(node, typeof(RdfLiteralNode));
			Assert.AreEqual("1230000000000", ((node as RdfLiteralNode).String as StringLiteralNode).Value);
			Assert.AreEqual("http://www.w3.org/2001/XMLSchema#double", ((node as RdfLiteralNode).IriRef as IriRefNode).Value);

			node = new NumericLiteralNode()
			{
				DecimalValue = 123.4M,
				Type = NumericLiteralNode.LiteralType.Decimal
			};
			explicateLiterals(ref node);
			Assert.IsInstanceOfType(node, typeof(RdfLiteralNode));
			Assert.AreEqual("123.4", ((node as RdfLiteralNode).String as StringLiteralNode).Value);
			Assert.AreEqual("http://www.w3.org/2001/XMLSchema#decimal", ((node as RdfLiteralNode).IriRef as IriRefNode).Value);

			node = new BooleanLiteralNode()
			{
				Value = true
			};
			explicateLiterals(ref node);
			Assert.IsInstanceOfType(node, typeof(RdfLiteralNode));
			Assert.AreEqual("true", ((node as RdfLiteralNode).String as StringLiteralNode).Value);
			Assert.AreEqual("http://www.w3.org/2001/XMLSchema#boolean", ((node as RdfLiteralNode).IriRef as IriRefNode).Value);

			node = new BooleanLiteralNode()
			{
				Value = false
			};
			explicateLiterals(ref node);
			Assert.IsInstanceOfType(node, typeof(RdfLiteralNode));
			Assert.AreEqual("false", ((node as RdfLiteralNode).String as StringLiteralNode).Value);
			Assert.AreEqual("http://www.w3.org/2001/XMLSchema#boolean", ((node as RdfLiteralNode).IriRef as IriRefNode).Value);
		}
	}
}
