﻿using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using SharQL.Ast;
using SharQL.Ast.Nodes;
using SharQL.Ast.Visitor;
using SharQL.Algebra.AstUtils.Visitor;
using SharQL.Algebra.Tests.Utils;

namespace SharQL.Algebra.AstUtils.Tests
{
    /// <summary>
    ///This is a test class for TriplesSameSubjectResolverVisitorTest and is 
    ///intended to contain all TriplesSameSubjectResolverVisitorTest Unit Tests
    ///</summary>
    [TestClass()]
    public class TriplesSameSubjectResolverVisitorTest
    {
        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        [TestMethod]
        public void TestTriplesSameSubject()
        {
            // Create AST subtree resembling triple block ":a :b :c ; :x :y"
            INode triplesBlockNode = new TriplesBlockNode(
                new TriplesSameSubjectNode(
                    new PrefixedNameNode() { Pn_Local = "a" },
                    null,
                    new PropertyListNode(
                        new PrefixedNameNode() { Pn_Local = "b" },
                        new ObjectListNode(
                            new PrefixedNameNode() { Pn_Local = "c" },
                            null
                        ),
                        new PropertyListNode(
                            new PrefixedNameNode() { Pn_Local = "x" },
                            new ObjectListNode(
                                new PrefixedNameNode() { Pn_Local = "y" },
                                null
                            ),
                            null
                        )
                    )
                ),
                null
            );

            TriplesSameSubjectResolverVisitor visitor = new TriplesSameSubjectResolverVisitor();
            Visualizer.Visualize.PrepareAst(triplesBlockNode);
            triplesBlockNode.Accept(visitor, VisitorOrder.ParentBeforeChildren);
            Visualizer.Visualize.CompareAst(triplesBlockNode);

            // Assert that the trailing PropertyListNode 
            Assert.IsTrue(triplesBlockNode.Children[0].Children[2].Children.Count == 3);

            // Assert the structure of the extended subtree
            Assert.IsTrue(triplesBlockNode.Children.Count == 2);

            Assert.IsTrue(triplesBlockNode.Children[1] is TriplesBlockNode);
            Assert.IsTrue(triplesBlockNode.Children[1].Children.Count == 2);
            
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0] is TriplesSameSubjectNode);
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children.Count == 3);
 
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[0] is PrefixedNameNode);
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[0].Children.Count == 0);
            Assert.IsTrue((triplesBlockNode.Children[1].Children[0].Children[0] as PrefixedNameNode).Pn_Local == "a");

            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[1] == null);

            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2] is PropertyListNode);
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children.Count == 3);

            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[0] is PrefixedNameNode);
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[0].Children.Count == 0);
            Assert.IsTrue((triplesBlockNode.Children[1].Children[0].Children[2].Children[0] as PrefixedNameNode).Pn_Local == "x");
            
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[1] is ObjectListNode);
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[1].Children.Count == 2);
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[1].Children[1] == null);

            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[1].Children[0] is PrefixedNameNode);
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[1].Children[0].Children.Count == 0);
            Assert.IsTrue((triplesBlockNode.Children[1].Children[0].Children[2].Children[1].Children[0] as PrefixedNameNode).Pn_Local == "y");

            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[2] == null);
            
            Assert.IsTrue(triplesBlockNode.Children[1].Children[1] == null);
        }

        [TestMethod]
        public void TestTriplesSameSubjectAndObject()
        {
            // Create AST subtree resembling triple block ":a :b :c , :x"
            INode triplesBlockNode = new TriplesBlockNode(
                new TriplesSameSubjectNode(
                    new PrefixedNameNode() { Pn_Local = "a" },
                    null,
                    new PropertyListNode(
                        new PrefixedNameNode() { Pn_Local = "b" },
                        new ObjectListNode(
                            new PrefixedNameNode() { Pn_Local = "c" },
                            new ObjectListNode(
                                new PrefixedNameNode() { Pn_Local = "x" },
                                null
                            )
                        ),
                        null
                    )
                ),
                null
            );

            TriplesSameSubjectResolverVisitor visitor = new TriplesSameSubjectResolverVisitor();
            INode oldTriplesBlockNode = triplesBlockNode.Clone() as NodeBase;
            triplesBlockNode.Accept(visitor, VisitorOrder.ParentBeforeChildren);
            Visualizer.Visualize.Ast(oldTriplesBlockNode, triplesBlockNode);

            // Assert that the trailing PropertyListNode 
            Assert.IsTrue(triplesBlockNode.Children[0].Children[2].Children.Count == 3);

            // Assert the structure of the extended subtree
            Assert.IsTrue(triplesBlockNode.Children.Count == 2);

            Assert.IsTrue(triplesBlockNode.Children[1] is TriplesBlockNode);
            Assert.IsTrue(triplesBlockNode.Children[1].Children.Count == 2);

            Assert.IsTrue(triplesBlockNode.Children[1].Children[0] is TriplesSameSubjectNode);
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children.Count == 3);

            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[0] is PrefixedNameNode);
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[0].Children.Count == 0);
            Assert.IsTrue((triplesBlockNode.Children[1].Children[0].Children[0] as PrefixedNameNode).Pn_Local == "a");

            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[1] == null);

            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2] is PropertyListNode);
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children.Count == 3);

            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[0] is PrefixedNameNode);
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[0].Children.Count == 0);
            Assert.IsTrue((triplesBlockNode.Children[1].Children[0].Children[2].Children[0] as PrefixedNameNode).Pn_Local == "b");

            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[1] is ObjectListNode);
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[1].Children.Count == 2);
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[1].Children[1] == null);

            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[1].Children[0] is PrefixedNameNode);
            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[1].Children[0].Children.Count == 0);
            Assert.IsTrue((triplesBlockNode.Children[1].Children[0].Children[2].Children[1].Children[0] as PrefixedNameNode).Pn_Local == "x");

            Assert.IsTrue(triplesBlockNode.Children[1].Children[0].Children[2].Children[2] == null);

            Assert.IsTrue(triplesBlockNode.Children[1].Children[1] == null);
        }

		[TestMethod]
		public void TestSeriesOfSameSubjectAndObject()
		{
			QueryNode root = createQueryWithSeriesOfSameSubjectAndObject();
            Visualizer.Visualize.PrepareAst(root);
			expandSameSubjectAndObject(ref root);
            Visualizer.Visualize.CompareAst(root);
			verifySeriesOfSameSubjectAndObject(root);
		}

		/// <summary>
		/// <code>
		/// SELECT *
		/// {
		/// ?a ?b ?c ; ?p ?q , ?r , ?s ; ?e ?f .
		/// }
		/// </code>
		/// </summary>
		private QueryNode createQueryWithSeriesOfSameSubjectAndObject()
		{
			return new QueryNode(
				new PrologueNode(null, null),
				new SelectQueryNode(
					new StarVarListNode() { IsStar = true },
					new WhereClauseNode(
						new GroupGraphPatternNode(
							new TriplesBlockNode(
								new TriplesSameSubjectNode(
									new VarNode("a"),
									null,
									new PropertyListNode(
										new VarNode("b"),
										new ObjectListNode(
											new VarNode("c"),
											null
										),
										new PropertyListNode(
											new VarNode("p"),
											new ObjectListNode(
												new VarNode("q"),
												new ObjectListNode(
													new VarNode("r"),
													new ObjectListNode(
														new VarNode("s"),
														null
													)
												)
											),
											new PropertyListNode(
												new VarNode("e"),
												new ObjectListNode(
													new VarNode("f"),
													null
												),
												null
											)
										)
									)
								),
								null
							),
							null
						)
					)
				)
			);
		}

		private void expandSameSubjectAndObject(ref QueryNode root)
		{
            root.Accept(new TriplesSameSubjectResolverVisitor(), VisitorOrder.ParentBeforeChildren);
		}

		private void verifySeriesOfSameSubjectAndObject(QueryNode root)
		{
			TriplesBlockNode[] triples = TripleUtils.FetchSubsequentTriples(root.Children[1].Children[1].Children[0].Children[0] as TriplesBlockNode);
			List<TriplesBlockNode> triplesList = new List<TriplesBlockNode>(triples);

			Assert.AreEqual(5, triplesList.Count);

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new VarNode("a"),
				new VarNode("b"),
				new VarNode("c")));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new VarNode("a"),
				new VarNode("p"),
				new VarNode("q")));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new VarNode("a"),
				new VarNode("p"),
				new VarNode("r")));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new VarNode("a"),
				new VarNode("p"),
				new VarNode("s")));

			Assert.IsTrue(TestUtils.TryRemoveFirstTriple(triplesList,
				new VarNode("a"),
				new VarNode("e"),
				new VarNode("f")));

			Assert.AreEqual(0, triplesList.Count);
		}
    }
}
