﻿using System.Collections.Generic;
using SharQL.Ast;
using SharQL.Ast.Nodes;

namespace SharQL.Algebra.Tests.Utils
{
	internal static class TestUtils
	{
		public static bool HasSameValues(INode node, INode realNode)
		{
			if (node.GetType() == typeof(PrefixedNameNode) && realNode.GetType() == typeof(PrefixedNameNode))
			{
				PrefixedNameNode pnnSubject = (PrefixedNameNode)node;
				PrefixedNameNode pnnRealSubject = (PrefixedNameNode)realNode;

				return pnnSubject.Pn_Local == pnnRealSubject.Pn_Local
					&& pnnSubject.Pname_Ns == pnnRealSubject.Pname_Ns;
			}
			else if (node.GetType() == typeof(VarNode) && realNode.GetType() == typeof(VarNode))
			{
				VarNode vnSubject = (VarNode)node;
				VarNode vnRealSubject = (VarNode)realNode;

				return vnSubject.Value == vnRealSubject.Value;
			}
			else if (node.GetType() == typeof(IriRefNode) && realNode.GetType() == typeof(IriRefNode))
			{
				IriRefNode irnSubject = (IriRefNode)node;
				IriRefNode irnRealSubject = (IriRefNode)realNode;

				return irnSubject.Value == irnRealSubject.Value;
			}
			else if (node.GetType() == typeof(RdfLiteralNode) && realNode.GetType() == typeof(RdfLiteralNode))
			{
				StringLiteralNode slnNode = ((RdfLiteralNode)node).String as StringLiteralNode;
				StringLiteralNode slnRealNode = ((RdfLiteralNode)realNode).String as StringLiteralNode;

				return slnNode.Value == slnRealNode.Value;
			}
			else if (node.GetType() == typeof(BlankNode) && realNode.GetType() == typeof(BlankNode))
			{
				return true; // ignore blank nodes for later verification
			}

			return false;
		}

		public static bool TryRemoveFirstTriple(List<TriplesBlockNode> triples, INode subject, INode predicate, INode @object)
		{
			for (int i = 0; i < triples.Count; i++)
			{
				INode realSubject = TripleUtils.GetSubject(triples[i]), realPredicate = TripleUtils.GetPredicate(triples[i]), realObject = TripleUtils.GetObject(triples[i]);

				if (subject.GetType() == realSubject.GetType()
					&& predicate.GetType() == realPredicate.GetType()
					&& @object.GetType() == realObject.GetType())
				{
					if (TestUtils.HasSameValues(subject, realSubject)
						&& TestUtils.HasSameValues(predicate, realPredicate)
						&& TestUtils.HasSameValues(@object, realObject))
					{
						triples.RemoveAt(i);
						return true;
					}
				}
			}
			return false;
		}
	}
}
