﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using SharQL.Ast.Nodes;

namespace SharQL.Algebra.QueryStructure.Tests
{
    /// <summary>
    /// Summary description for DistinctReducedTest
    /// </summary>
    [TestClass]
    public class DistinctReducedTest
    {
        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void TestDistinctReducedModifierNotPresent()
        {
            Parser parser = new Parser();
            parser.Parse("SELECT * {}");
            Query query = QueryTransformer.Transform(parser.AstRoot as QueryNode);

            SelectQueryDescription description = query.Description as SelectQueryDescription;
            Assert.AreEqual(DistinctReducedModifiers.None, description.DistinctReducedModifier);
        }

        [TestMethod]
        public void TestDistinctModifier()
        {
            Parser parser = new Parser();
            parser.Parse("SELECT DISTINCT * {}");
            Query query = QueryTransformer.Transform(parser.AstRoot as QueryNode);

            SelectQueryDescription description = query.Description as SelectQueryDescription;
            Assert.AreEqual(DistinctReducedModifiers.Distinct, description.DistinctReducedModifier);
        }

        [TestMethod]
        public void TestReducedModifier()
        {
            Parser parser = new Parser();
            parser.Parse("SELECT REDUCED * {}");
            Query query = QueryTransformer.Transform(parser.AstRoot as QueryNode);

            SelectQueryDescription description = query.Description as SelectQueryDescription;
            Assert.AreEqual(DistinctReducedModifiers.Reduced, description.DistinctReducedModifier);
        }
    }
}
