﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using SharQL.Ast.Nodes;

namespace SharQL.Algebra.QueryStructure.Tests
{
    /// <summary>
    /// Summary description for PrologueTest
    /// </summary>
    [TestClass]
    public class PrologueTest
    {
        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void TestPrologueNotPresent()
        {
            Parser parser = new Parser();
            parser.Parse("SELECT * {}");
            Query query = QueryTransformer.Transform(parser.AstRoot as QueryNode);

            Assert.IsNull(query.Base);
            Assert.AreEqual(0, query.Prefixes.Count);
        }

        [TestMethod]
        public void TestProloguePresent()
        {
            Parser parser = new Parser();
            parser.Parse("BASE <http://abc/> PREFIX a: <def/> PREFIX b: <http://ghi/> SELECT * {}");
            Query query = QueryTransformer.Transform(parser.AstRoot as QueryNode);

			Assert.AreEqual("http://abc/", query.Base);
            Assert.AreEqual(2, query.Prefixes.Count);
            Assert.IsTrue(query.Prefixes.ContainsKey("a"));
            Assert.IsTrue(query.Prefixes.ContainsKey("b"));
			Assert.AreEqual("http://abc/def/", query.Prefixes["a"]);
			Assert.AreEqual("http://ghi/", query.Prefixes["b"]);
        }

		//[TestMethod]
		//public void TestComplexPrologue()
		//{
		//    Parser parser = new Parser();
		//    parser.Parse("PREFIX abc: <http://example.com/exampleOntology#> SELECT ?capital ?countryWHERE {  ?x abc:cityname ?capital ;     abc:isCapitalOf ?y .  ?y abc:countryname ?country ;     abc:isInContinent abc:Africa .}");
		//    Query query = QueryTranslator.Translate(parser.AstRoot as QueryNode);
		//}
    }
}
