﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using SharQL.Ast.Nodes;

namespace SharQL.Algebra.QueryStructure.Tests
{
    /// <summary>
    /// Summary description for SharedSubjectTripleTest
    /// </summary>
    [TestClass]
    public class SharedSubjectTripleTest
    {
        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void TestSharedSubjectAndPredicate()
        {
            Parser parser = new Parser();
            Assert.IsTrue(parser.Parse(@"PREFIX : <http://example.org/ns#> 
PREFIX xsd:        <http://www.w3.org/2001/XMLSchema#> 

SELECT ?s WHERE {
 ?s :p1 1, 2 .
}"));
            Query query = QueryTransformer.Transform(parser.AstRoot as QueryNode);
            SelectQueryDescription description = query.Description as SelectQueryDescription;

            Pattern whereClause = description.WhereClause;
            Assert.IsInstanceOfType(whereClause, typeof(Join));
            Assert.IsInstanceOfType(((Join)whereClause).FirstPattern, typeof(BasicGraphPattern));
            Assert.IsInstanceOfType(((Join)whereClause).SecondPattern, typeof(BasicGraphPattern));
            BasicGraphPattern bgp1 = ((Join)whereClause).FirstPattern as BasicGraphPattern;
            BasicGraphPattern bgp2 = ((Join)whereClause).SecondPattern as BasicGraphPattern;

            Assert.AreEqual(bgp1.Triples.Count, 1);
            Assert.IsInstanceOfType(bgp1.Triples[0].Subject, typeof(Resource));
            Assert.IsInstanceOfType(bgp1.Triples[0].Predicate, typeof(Resource));
            Assert.IsInstanceOfType(bgp1.Triples[0].Object, typeof(Resource));
            Assert.AreEqual(bgp1.Triples[0].Subject.Type, ResourceType.Variable);
            Assert.AreEqual(bgp1.Triples[0].Predicate.Type, ResourceType.Iri);
            Assert.AreEqual(bgp1.Triples[0].Object.Type, ResourceType.Literal);
            Assert.AreEqual(bgp1.Triples[0].Object.Value, "\"1\"^^http://www.w3.org/2001/XMLSchema#integer");

            Assert.AreEqual(bgp2.Triples.Count, 1);
            Assert.IsInstanceOfType(bgp2.Triples[0].Subject, typeof(Resource));
            Assert.IsInstanceOfType(bgp2.Triples[0].Predicate, typeof(Resource));
            Assert.IsInstanceOfType(bgp2.Triples[0].Object, typeof(Resource));
            Assert.AreEqual(bgp2.Triples[0].Subject.Type, ResourceType.Variable);
            Assert.AreEqual(bgp2.Triples[0].Predicate.Type, ResourceType.Iri);
            Assert.AreEqual(bgp2.Triples[0].Object.Type, ResourceType.Literal);
            Assert.AreEqual(bgp2.Triples[0].Object.Value, "\"2\"^^http://www.w3.org/2001/XMLSchema#integer");
        }
    }
}
