﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using SharQL.Ast.Nodes;
using SharQL.Ast;

namespace SharQL.Algebra.QueryStructure.Tests
{
	/// <summary>
	/// Summary description for VariablesInScopeTest
	/// </summary>
	[TestClass]
	public class VariablesInScopeTest
	{
		public VariablesInScopeTest()
		{
		}

		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		//
		// You can use the following additional attributes as you write your tests:
		//
		// Use ClassInitialize to run code before running the first test in the class
		// [ClassInitialize()]
		// public static void MyClassInitialize(TestContext testContext) { }
		//
		// Use ClassCleanup to run code after all tests in a class have run
		// [ClassCleanup()]
		// public static void MyClassCleanup() { }
		//
		// Use TestInitialize to run code before running each test 
		// [TestInitialize()]
		// public void MyTestInitialize() { }
		//
		// Use TestCleanup to run code after each test has run
		// [TestCleanup()]
		// public void MyTestCleanup() { }
		//
		#endregion

		[TestMethod]
		public void TestVariablesInScope1()
		{
			Parser p = new Parser();
			string query = 
				@"SELECT *
				{
					<a:x> <a:p> ?v .
					{
						?a ?b ?w
						OPTIONAL
						{
							?b ?c ?w
						}
					}
				}";

			p.Parse(query);
			WhereClauseNode whereClause = getWhereClauseFromQuery(p.AstRoot);
			Pattern result = WhereClauseTransformer.Transform(whereClause);

			Assert.AreEqual(5, result.VariablesInScope.Count);
			CollectionAssert.AllItemsAreNotNull(result.VariablesInScope);
			CollectionAssert.AllItemsAreUnique(result.VariablesInScope);
			CollectionAssert.AllItemsAreInstancesOfType(result.VariablesInScope, typeof(string));
			CollectionAssert.Contains(result.VariablesInScope, "a");
			CollectionAssert.Contains(result.VariablesInScope, "b");
			CollectionAssert.Contains(result.VariablesInScope, "c");
			CollectionAssert.Contains(result.VariablesInScope, "v");
			CollectionAssert.Contains(result.VariablesInScope, "w");
		}

		private WhereClauseNode getWhereClauseFromQuery(INode query)
		{
			return ((SelectQueryNode)query.Children[1]).WhereClause as WhereClauseNode;
		}
	}
}
