﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using SharQL.Ast;
using SharQL.Ast.Nodes;
using SharQL.Ast.Visitor;

namespace SharQL.Algebra.Visualizer
{
	internal class TreeBuilderVisitor : IVisitor
	{
		/// <summary>Whether to generate nodes for null values.</summary>
		public bool ShowNulls { get; set; }

		/// <summary>Gets the map of tree nodes created.</summary>
		public Dictionary<INode, TreeNode> TreeNodes { get; private set; }

		/// <summary>Indexer required for visitor support.</summary>
		/// <param name="nodeType">Ignored</param>
		/// <returns>A delegate for translating AST node into 
		/// TreeNode object.</returns>
		public Action<INode> this[Type nodeType] { get { return constructTreeNode; } }

		/// <summary>Constructor</summary>
		public TreeBuilderVisitor()
		{
			TreeNodes = new Dictionary<INode, TreeNode>();
			ShowNulls = false;
		}

		private void constructTreeNode(INode node)
		{
			NodeBase nb = node as NodeBase;
			if (nb != null)
			{
				// TreeNodes maps INode objects to TreeNode objects
				TreeNodes[node] = new TreeNode(nb.ToString().Split('.').Last() + getExtendedData(node));
				TreeNodes[node].Tag = node;
				foreach (INode child in nb.Children)
				{
					if (child != null && TreeNodes.ContainsKey(child))
					{
						TreeNodes[node].Nodes.Add(TreeNodes[child]);
					}
					else if (ShowNulls)
					{
						TreeNodes[node].Nodes.Add(new TreeNode("null"));
					}
				}
			}
		}

		private string getExtendedData(INode node)
		{
			string format = " ({0})";
			if (node is IriRefNode)
			{
				return string.Format(format, ((IriRefNode)node).Value);
			}
			if (node is VarNode)
			{
				return string.Format(format, ((VarNode)node).Value);
			}
			if (node is PrefixedNameNode)
			{
				return string.Format(format, ((PrefixedNameNode)node).Pname_Ns + ":" + ((PrefixedNameNode)node).Pn_Local);
			}
			if (node is BlankNode)
			{
				return string.Format(format, ((BlankNode)node).Value);
			}
			if (node is StringLiteralNode)
			{
				return string.Format(format, ((StringLiteralNode)node).Value);
			}
			return string.Empty;
		}
	}
}
