﻿
using SharQL.Ast;

namespace SharQL.Algebra.AstUtils
{
    internal static class Util
    {
        /// <summary>
        /// Replaces a node in the AST.
        /// </summary>
        /// <param name="oldNode">Node to be replaced.</param>
        /// <param name="newNode">Replacement node.</param>
        public static void ReplaceNode(INode oldNode, INode newNode)
        {
            ReplaceNode(oldNode, newNode, false);
        }

        /// <summary>
        /// Replaces a node in the AST.
        /// </summary>
        /// <param name="oldNode">Node to be replaced.</param>
        /// <param name="newNode">Replacement node.</param>
        /// <param name="includeChildren">Whether or not to add the old node's
        /// children to the new node.</param>
        public static void ReplaceNode(INode oldNode, INode newNode, bool includeChildren)
        {
            int pos = oldNode.Parent.Children.IndexOf(oldNode);

            oldNode.Parent.Children.RemoveAt(pos);
            oldNode.Parent.Children.Insert(pos, newNode);

            newNode.Parent = oldNode.Parent;

            if (includeChildren)
            {
                foreach (INode child in oldNode.Children)
                {
                    newNode.Children.Add(child);
                }
            }
        }

        /// <summary>
        /// Inserts a triple block in a sequence of triple blocks.
        /// </summary>
        /// <param name="tripleBlockBefore">The triple block after which the new
        /// triple block is to be inserted.</param>
        /// <param name="tripleBlock">The new triple block to be inserted.</param>
        public static void InsertTripleBlock(INode tripleBlockBefore, INode tripleBlock)
        {
            INode currentChild = tripleBlockBefore.Children[1];
            tripleBlockBefore.Children.RemoveAt(1);
            tripleBlockBefore.Children.Insert(1, tripleBlock);
            tripleBlock.Parent = tripleBlockBefore;
            if (currentChild != null)
            {
                tripleBlock.Children.RemoveAt(1);
                tripleBlock.Children.Insert(1, currentChild);
                currentChild.Parent = tripleBlock;
            }
        }
    }
}
