﻿using System;
using SharQL.Ast;
using SharQL.Ast.Nodes;
using SharQL.Ast.Visitor;

namespace SharQL.Algebra.AstUtils.Visitor
{
    internal class BlankNodeReificationVisitor : IVisitor
    {
        /// <summary>Indexer required for visitor support.</summary>
        /// <param name="nodeType">Ignored</param>
        /// <returns>A delegate for translating syntactic sugar for triple block
        /// blank subject elements into its core representation.</returns>
        public Action<INode> this[Type nodeType]
        {
            get
            {
                if (nodeType == typeof(BlankNode))
                {
                    return identifyBlankNode;
                }
                else if (nodeType == typeof(BlankNodePropertyListNode))
                {
                    return reificateBlankNode;
                }

                return null;
            }
        }

        private void identifyBlankNode(INode node)
        {
            BlankNode blankNode = node as BlankNode;

            if (blankNode.Value == null)
            {
                blankNode.Value = Guid.NewGuid().ToString();
            }
        }

        private void reificateBlankNode(INode node)
        {
            INode blankNode = new BlankNode() { Value = Guid.NewGuid().ToString() };
            
            TriplesSameSubjectResolverVisitor visitor = new TriplesSameSubjectResolverVisitor();
            visitor.SubjectNode = blankNode;
            node.Accept(visitor, VisitorOrder.ParentBeforeChildren);

            Util.ReplaceNode(node, blankNode);
        }
    }
}
