﻿using System;

using SharQL.Ast;
using SharQL.Ast.Nodes;
using SharQL.Ast.Visitor;

namespace SharQL.Algebra.AstUtils.Visitor
{
    internal class TriplesSameSubjectResolverVisitor : IVisitor
    {
        public INode SubjectNode { get; internal set; }

        /// <summary>Indexer required for visitor support.</summary>
        /// <param name="nodeType">Ignored</param>
        /// <returns>A delegate for translating syntactic sugar for triple blocks
        /// into its core representation.</returns>
        public Action<INode> this[Type nodeType] 
        { 
            get 
            {
                if (nodeType == typeof(PropertyListNode))
                {
                    return reifyMixedTriples;
                }
                else if (nodeType == typeof(BlankNodePropertyListNode))
                {
                    return reifyMixedTriples;
                }

                return null;
            } 
        }

        private void reifyObjectNodes(INode subjectNode, INode currentObjectNode, INode tripleBlock)
        {
            // Reify triples with shared subject and object 
            while (currentObjectNode != null)
            {
                currentObjectNode.Parent.Children[1] = null;

                INode newObjectTripleBlock =
                    new TriplesBlockNode(
                        new TriplesSameSubjectNode(
                            subjectNode.Clone() as NodeBase,
                            null,
                            new PropertyListNode(
                                currentObjectNode.Parent.Parent.Children[0] as NodeBase,
                                currentObjectNode as NodeBase,
                                null)
                            ),
                            null
                    );

                currentObjectNode = currentObjectNode.Children[1];

                Util.InsertTripleBlock(tripleBlock, newObjectTripleBlock);
                tripleBlock = newObjectTripleBlock;
            }
        }

        private void reifyMixedTriples(INode node)
        {
            // Find anchestor TriplesBlockNode
            INode tripleBlock = node;
            while (tripleBlock.GetType() != typeof(TriplesBlockNode))
            {
                tripleBlock = tripleBlock.Parent;
            }

            // Choose either a given subject node or the one belonging to
            // the current triple block if no subject node is given in advance
            INode subjectNode = SubjectNode ?? node.Parent.Children[0];

            if (node is BlankNodePropertyListNode)
            {
                if (node.Parent is TriplesSameSubjectNode &&
                    node.Parent.Children[2] == null)
                {
                    node.Parent.Children[2] = node.Children[0];
                }
                else
                {
                    INode newTripleBlock =
                        new TriplesBlockNode(
                            new TriplesSameSubjectNode(
                                subjectNode.Clone() as NodeBase,
                                null,
                                node.Children[0] as NodeBase
                                ),
                                null
                        );

                    Util.InsertTripleBlock(tripleBlock, newTripleBlock);
                }

                node = node.Children[0];
            }

            // Reify triples with shared subject and object
            reifyObjectNodes(subjectNode, node.Children[1].Children[1], tripleBlock);

            // Reify triples with shared subject
            INode currentPropertyNode = node.Children[2];
            while (currentPropertyNode != null)
            {
                currentPropertyNode.Parent.Children[2] = null;

                INode newTripleBlock =
                    new TriplesBlockNode(
                        new TriplesSameSubjectNode(
                            subjectNode.Clone() as NodeBase,
                            null,
                            currentPropertyNode as NodeBase
                            ),
                            null
                    );

                // Reify triples with shared subject and object
                reifyObjectNodes(subjectNode, currentPropertyNode.Children[1].Children[1], tripleBlock);

                currentPropertyNode = currentPropertyNode.Children[2];

                Util.InsertTripleBlock(tripleBlock, newTripleBlock);
                tripleBlock = newTripleBlock;
            }
        }
    }
}
