﻿using System.Collections.Generic;
using SharQL.Ast;
using SharQL.Ast.Nodes;

namespace SharQL.Algebra
{
    /// <summary>
    /// Static class for normalizing AST patterns to the form required by the
	/// W3C's approach of transformation into SPARQL algebra.
    /// </summary>
	internal static class PatternNormalizer
	{
        /// <summary>
        /// Normalizes the group graph pattern, flattening the list of children and
		/// removing surrogate list headers.
        /// </summary>
        /// <param name="groupGraphPattern">The group graph pattern to normalize.</param>
		public static void NormalizeGroupGraphPattern(GroupGraphPatternNode groupGraphPattern)
		{
			List<INode> newChildren = new List<INode>();
			GroupGraphPatternBlockListNode patternList = groupGraphPattern.Children[1] as GroupGraphPatternBlockListNode;
			GroupGraphPatternBlockNode patternBlock = null;

			newChildren.Add(groupGraphPattern.Children[0]);

			while (patternList != null)
			{
				patternBlock = patternList.Children[0] as GroupGraphPatternBlockNode;
				if (patternBlock.Children[0] != null)
				{
					if (patternBlock.Children[0] is GroupOrUnionGraphPatternNode)
					{
						NormalizeGroupOrUnionGraphPattern(patternBlock.Children[0] as GroupOrUnionGraphPatternNode);
						newChildren.Add(patternBlock.Children[0]);
					}
					else
					{
						newChildren.Add(patternBlock.Children[0]);
					}
				}
				if (patternBlock.Children[1] != null)
				{
					newChildren.Add(patternBlock.Children[1]);
				}
				if (patternBlock.Children[2] != null)
				{
					newChildren.Add(patternBlock.Children[2]);
				}
				patternList = patternList.Children[1] as GroupGraphPatternBlockListNode;
			}

			groupGraphPattern.Children.Clear();
			foreach (INode child in newChildren)
			{
				groupGraphPattern.Children.Add(child);
			}
		}

        /// <summary>
		/// Normalizes the group or union graph pattern, flattening the list of children.
        /// </summary>
        /// <param name="groupOrUnionGraphPattern">The group or union graph pattern to
		/// normalize.</param>
		public static void NormalizeGroupOrUnionGraphPattern(GroupOrUnionGraphPatternNode groupOrUnionGraphPattern)
		{
			List<INode> newChildren = new List<INode>();
			GroupGraphPatternUnionListNode unionList = groupOrUnionGraphPattern.Children[1] as GroupGraphPatternUnionListNode;

			newChildren.Add(groupOrUnionGraphPattern.Children[0]);

			while (unionList != null)
			{
				newChildren.Add(unionList.Children[0]);
				unionList = unionList.Children[1] as GroupGraphPatternUnionListNode;
			}

			groupOrUnionGraphPattern.Children.Clear();
			foreach (INode child in newChildren)
			{
				groupOrUnionGraphPattern.Children.Add(child);
			}
		}
	}
}
