﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Microsoft.Ceres.Evaluation.Operators;
using Microsoft.Ceres.Evaluation.Operators.BuiltIn.Basic;
using Microsoft.Ceres.Evaluation.Operators.BuiltIn.Join;
using Microsoft.Ceres.Evaluation.Operators.Graphs;
using Microsoft.Ceres.SearchCore.Search.Operators;

namespace SharQL.Component.Dumpers
{
    public class OperatorDumper
    {
        private TextWriter _writer;

        private OperatorDumper()
        { }

        public static void Write(OperatorBase node)
        {
            Write(node, Console.Out);
        }

        public static void Write(OperatorBase node, TextWriter writer)
        {
            OperatorDumper dumper = new OperatorDumper();
            dumper._writer = writer;
            dumper.WriteObject(null, node);
        }

        public void WriteObject(string prefix, OperatorBase node)
        {
            if (node == null) return;

            _writer.Write(prefix ?? string.Empty);
            _writer.Write(node.GetType().Name);

            if (node is TrimOperator)
            {
                _writer.WriteLine(string.Format(" (Hits = {0}, Offset = {1})", ((TrimOperator)node).Hits, ((TrimOperator)node).Offset));
            }
            else if (node is MapOperator)
            {
                IEnumerable<string> mappings = ((MapOperator)node).ParameterMap.Select((kvp, i) => string.Format("{0} => {1}", kvp.Key, kvp.Value));
                _writer.WriteLine(" (" + string.Join(", ", mappings.ToArray()) + ")");
            }
            else if (node is SortOperator)
            {
                IEnumerable<string> fields = ((SortOperator)node).SortFields.Select((sd, i) => string.Format("{0}({1})", sd.Field, sd.NaturalOrder ? "ASC" : "DESC"));
                _writer.WriteLine(" (" + string.Join(", ", fields.ToArray()) + ")");
            }
            else if (node is MergeJoinOperator)
            {
				int joinPrefix = ((MergeJoinOperator)node).JoinPrefix;
				if (joinPrefix == 0)
				{
					// Cartesian product
					_writer.WriteLine(" (Cartesian product)");
				}
				else
				{
					// Natural join
					_writer.Write(" (Natural join on ");
					List<string> keys = new List<string>();
					for (int i = 0; i < joinPrefix; ++i)
					{
						keys.Add(((MergeJoinOperator)node).Inputs[0].RecordSetType.RecordType.Name(i));
					}
					_writer.WriteLine("{0})", string.Join(", ", keys.ToArray()));
				}
            }
            else if (node is LookupOperator)
            {
                _writer.WriteLine(" ({0})", ((LookupOperator)node).Word);
            }
			else if (node is SelectOperator)
			{
				_writer.WriteLine(" ({0})", ((SelectOperator)node).Filter);
			}
			else if (node is ProjectDistinctOperator)
			{
				_writer.WriteLine(" (Prefix: {0})", ((ProjectDistinctOperator)node).Prefix);
			}
			else
            {
                _writer.Write(Environment.NewLine);
            }

            foreach (Edge edge in node.Inputs)
            {
                WriteObject((prefix ?? string.Empty) + "  ", edge.Source);
            }
        }
    }
}
