﻿using Microsoft.Ceres.CoreServices.Services.Configuration;
namespace SharQL.Component
{
	/// <summary>
	/// Contains configuration parameters for the SPARQL Query Service.
	/// </summary>
	public interface ISparqlQueryServiceConfiguration
	{
		/// <summary>
		/// Gets the name of the occurrence index.
		/// </summary>
		/// <value>The name of the occurrence index.</value>
        [ConfigProperty(DefaultValue="Occurrence2")]
        string OccurrenceIndex { get; }

		/// <summary>
		/// Gets the name of the first value field.
		/// </summary>
		/// <value>The name of the first value field.</value>
        [ConfigProperty(DefaultValue = "Value1")]
        string FirstValueField { get; }

		/// <summary>
		/// Gets the name of the second value field.
		/// </summary>
		/// <value>The name of the second value field.</value>
        [ConfigProperty(DefaultValue = "Value2")]
        string SecondValueField { get; }

		/// <summary>
		/// Gets the prefix of subject nodes.
		/// </summary>
		/// <value>The prefix of subject nodes.</value>
        [ConfigProperty(DefaultValue = "s:")]
        string SubjectPrefix { get; }

		/// <summary>
		/// Gets the prefix of predicate nodes.
		/// </summary>
		/// <value>The prefix of predicate nodes.</value>
        [ConfigProperty(DefaultValue = "p:")]
        string PredicatePrefix { get; }

		/// <summary>
		/// Gets the prefix of object nodes.
		/// </summary>
		/// <value>The prefix of object nodes.</value>
        [ConfigProperty(DefaultValue = "o:")]
        string ObjectPrefix { get; }
	}
}
