﻿using System;
using System.IO;
using SharQL.Ast;
using SharQL.Ast.Nodes;
using Microsoft.Ceres.Evaluation.Operators;
using Microsoft.Ceres.Evaluation.Operators.Graphs;
using Microsoft.Ceres.Evaluation.Operators.BuiltIn.Basic;
using System.Collections.Generic;
using Microsoft.Ceres.Evaluation.Operators.BuiltIn.Join;
using Microsoft.Ceres.SearchCore.Search.Operators;

namespace SharQL.Component
{
    public class OperatorDumper
    {
        private TextWriter _writer;

        private OperatorDumper()
        { }

        public static void Write(OperatorBase node)
        {
            Write(node, Console.Out);
        }

        public static void Write(OperatorBase node, TextWriter writer)
        {
            OperatorDumper dumper = new OperatorDumper();
            dumper._writer = writer;
            dumper.WriteObject(null, node);
        }

        public void WriteObject(string prefix, OperatorBase node)
        {
            if (node == null) return;

            _writer.Write(prefix ?? string.Empty);
            _writer.Write(node.GetType().Name);

            if (node is TrimOperator)
            {
                _writer.WriteLine(string.Format("(Hits = {0}, Offset = {1})", ((TrimOperator)node).Hits, ((TrimOperator)node).Offset));
            }
            else if (node is MapOperator)
            {
                _writer.WriteLine(((MapOperator)node).RetainInput ? "(RetainInput = true)" : "(RetainInput = false)");
                foreach (var mapping in ((MapOperator)node).ParameterMap)
                {
                    _writer.WriteLine((prefix ?? string.Empty) + "    {0} => {1}", mapping.Key, mapping.Value);
                }
            }
            else if (node is SortOperator)
            {
                _writer.Write(Environment.NewLine);
                foreach (var field in ((SortOperator)node).SortFields)
                {
                    _writer.WriteLine((prefix ?? string.Empty) + "    {0} ({1})", field.Field, field.NaturalOrder ? "ASC" : "DESC");
                }
            }
            else if (node is MergeJoinOperator)
            {
                string firstJoinField = ((MergeJoinOperator)node).Inputs[0].RecordSetType.RecordType.Name(0);



                _writer.WriteLine(" ({0})", ((MergeJoinOperator)node).JoinPrefix == 0 ? "Cartesian product" : "Join: ...");
                _writer.WriteLine("{0}", ((MergeJoinOperator)node).Inputs[0].RecordSetType.RecordType.Name(0));
            }
            else if (node is LookupOperator)
            {
                _writer.WriteLine(" ({0})", ((LookupOperator)node).Word);
            }
			else if (node is SelectOperator)
			{
				_writer.WriteLine(" ({0})", ((SelectOperator)node).Filter);
			}
			else if (node is ProjectDistinctOperator)
			{
				_writer.WriteLine(" (Prefix: {0})", ((ProjectDistinctOperator)node).Prefix);
			}
			else
            {
                _writer.Write(Environment.NewLine);
            }

            foreach (Edge edge in node.Inputs)
            {
                WriteObject((prefix ?? string.Empty) + "  ", edge.Source);
            }
        }

        private void writeObject(string prefix, KeyValuePair<IdentifierProperty, ExpressionProperty> mapping)
        {
            
        }
    }
}
