﻿using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using Microsoft.Ceres.Evaluation.DataModel;
using Microsoft.Ceres.Evaluation.Services;

namespace SharQL.Component
{

    /// <summary>
    /// Class for generating the response.
    /// </summary>
    internal class OutputXmlGenerator
    {
        private readonly XDocument document = new XDocument();

        public OutputXmlGenerator()
        {
            document.Add(new XElement("context"));
        }

        public void Failure(string message)
        {
            document.Root.Add(
                new XElement("failure",
                    new XElement("message",
                        new XText(message)
                    )
                )
            );
        }

        public void Info(string message)
        {
            document.Root.Add(
                new XElement("info",
                    new XElement("message",
                        new XText(message)
                    )
                )
            );
        }

        public void FetchMainResult(IDictionary<string, IOutputRecordSet> results)
        {
            IOutputRecordSet mainResult;

            if (results == null)
            {
                Failure("Results was null!");
                return;
            }

            results.TryGetValue("Primary", out mainResult);
            if (mainResult != null)
            {
                results.Remove("Primary");
            }
            else
            {
                // No main result
                Failure("No main result!");
                return;
            }

            var resultElement = new XElement("result");
            document.Root.Add(resultElement);

            // The MergeJoin operator adds a prefix to the field name. We remove this prefix here.
            var recordType = mainResult.Properties.RecordSetType.RecordType;
            IList<string> names = recordType.Select(f => f.Name.Split('.').Last()).ToList();

            while (mainResult.MoveNext())
            {
                IRecord record = mainResult.Current;

                // Do not print out flush-records. These should be disregarded.
                if (record.Metadata.FlushRecord)
                {
                    continue;
                }

                var entryElement = new XElement("entry");
                resultElement.Add(entryElement);
           
                for (int i = 0; i < record.Count; i++)
                {
                    var fieldElement = new XElement(
                            (names[i]),
                            new XText(record[i].ToString()));
                    entryElement.Add(fieldElement);
                }
            }

            mainResult.Close();
        }

        public override string ToString()
        {
            return document.ToString();
        }

        /*public void QueryTerms(KeywordExpression expression)
        {
            if (document == null || document.DocumentElement == null)
            {
                return;
            }

            var queryTerms = document.CreateElement("queryterms");
            document.DocumentElement.AppendChild(queryTerms);

            for (int i = 0; i < expression.Count; i++)
            {
                var term = document.CreateElement("term");
                queryTerms.AppendChild(term);
                var value = document.CreateElement("value");

                var valueText = new StringBuilder();
                var termEx = expression[i] as TermExpression;
                if (termEx != null)
                {
                    valueText.Append(termEx.Term);
                }
                else
                {
                    var phraseEx = expression[i] as PhraseExpression;
                    if (phraseEx != null)
                    {
                        for (int j = 0; j < phraseEx.Count; j++)
                        {
                            valueText.Append(phraseEx[j].Term);
                            if (j != (phraseEx.Count - 1))
                            {
                                valueText.Append(' ');
                            }
                        }
                    }
                }

                value.AppendChild(document.CreateTextNode(valueText.ToString()));
                term.AppendChild(value);

                var scope = document.CreateElement("scope");
                if (expression[i].Scope != null)
                {
                    scope.AppendChild(document.CreateTextNode(expression[i].Scope));
                }

                term.AppendChild(scope);
            }
        }

        public void Navigators(IDictionary<string, IOutputRecordSet> results)
        {
            if (document == null || document.DocumentElement == null)
            {
                return;
            }

            var navigators = document.CreateElement("navigators");
            document.DocumentElement.AppendChild(navigators);

            foreach (KeyValuePair<string, IOutputRecordSet> navigator in results)
            {
                var nav = document.CreateElement("navigator");
                nav.SetAttribute("name", navigator.Key);
                navigators.AppendChild(nav);
                var recordType = navigator.Value.Descriptor.Type.RecordType;

                if (recordType.Count == 2)
                {
                    // Value navigator
                    nav.SetAttribute("type", "Value");

                    int countPos = recordType.Position(CountFieldName);
                    int valuePos = 1 - countPos;

                    while (navigator.Value.MoveNext())
                    {
                        var entry = document.CreateElement("entry");
                        nav.AppendChild(entry);

                        IRecord record = navigator.Value.Current;
                        var value = document.CreateElement("value");
                        value.AppendChild(document.CreateTextNode(record[valuePos].ToString()));
                        entry.AppendChild(value);
                        var count = document.CreateElement("count");
                        count.AppendChild(document.CreateTextNode(record[countPos].ToString()));
                        entry.AppendChild(count);
                    }
                }
                else
                {
                    // Range navigator
                    nav.SetAttribute("type", "Range");

                    int countPos = recordType.Position(CountFieldName);
                    int upperLimitPos = recordType.Position(UpperLimitFieldName);
                    int lowerLimitPos = recordType.Position(LowerLimitFieldName);

                    while (navigator.Value.MoveNext())
                    {
                        var entry = document.CreateElement("entry");
                        nav.AppendChild(entry);

                        IRecord record = navigator.Value.Current;
                        var lowerLimit = document.CreateElement("LowerLimit");
                        lowerLimit.AppendChild(document.CreateTextNode(record[lowerLimitPos].ToString()));
                        entry.AppendChild(lowerLimit);
                        var upperLimit = document.CreateElement("UpperLimit");
                        upperLimit.AppendChild(document.CreateTextNode(record[upperLimitPos].ToString()));
                        entry.AppendChild(upperLimit);
                        var count = document.CreateElement("count");
                        count.AppendChild(document.CreateTextNode(record[countPos].ToString()));
                        entry.AppendChild(count);
                    }
                }

                navigator.Value.Close();
            }
        }

        // The name of the count field in navigator results
        private const string CountFieldName = "Count";

        private const string UpperLimitFieldName = "UpperLimit";

        private const string LowerLimitFieldName = "LowerLimit";*/
    }
}
