﻿using System.Collections.Generic;
using SharQL.Ast.Visitor;
using System;

namespace SharQL.Ast
{
	/// <summary>
	/// The interface for all AST nodes created by the SharQL parser.
	/// </summary>
	public interface INode : ICloneable
	{
		/// <summary>
		/// Gets or sets the parent of the node. Should be set to
		/// <c>null</c> if node is root.
		/// </summary>
		INode Parent { get; set; }

		/// <summary>
		/// Gets the list of children of the node.
		/// </summary>
		IList<INode> Children { get; }

		/// <summary>
		/// Accepts a visitor. The visitor pattern can be used for
		/// e.g. optimizing the AST.
		/// </summary>
		/// <param name="visitor">The visitor</param>
		void Accept(IVisitor visitor);

		/// <summary>
		/// Accepts a visitor. The visitor pattern can be used for
		/// e.g. optimizing the AST.
		/// </summary>
		/// <param name="visitor">The visitor</param>
		/// <param name="order">The visitor order</param>
		void Accept(IVisitor visitor, VisitorOrder order);
	}
}
