﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using SharQL.Utils;

namespace SharQLTest
{
    /// <summary>
    ///This is a test class for PreprocessorTest and is intended
    ///to contain all PreprocessorTest Unit Tests
    ///</summary>
	[TestClass()]
	public class EscapeSequenceResolverTest
	{
		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		/// <summary>A test for ResolveCodepointEscapeSequences</summary>
		[TestMethod]
		public void ResolveCodepointEscapeSequencesTest()
		{
			string escapedString = @"\u0061";
			string expected = "a";
			string actual;
			actual = EscapeSequenceResolver.ResolveCodepointEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);

			escapedString = @"\U00000061";
			actual = EscapeSequenceResolver.ResolveCodepointEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);

			escapedString = @"ab\u0063";
			expected = "abc";
			actual = EscapeSequenceResolver.ResolveCodepointEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);

			escapedString = @"ab\\u0063";
			expected = @"ab\\u0063";
			actual = EscapeSequenceResolver.ResolveCodepointEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);

			escapedString = @"ab\\\u0063";
			expected = @"ab\\c";
			actual = EscapeSequenceResolver.ResolveCodepointEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);

			escapedString = @"\u00ae 2008";
			expected = @"® 2008";
			actual = EscapeSequenceResolver.ResolveCodepointEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);

			escapedString = @"\U000000AE 2008";
			actual = EscapeSequenceResolver.ResolveCodepointEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);

			escapedString = @"\u0061\u0062\u0063";
			expected = @"abc";
			actual = EscapeSequenceResolver.ResolveCodepointEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);
		}

		[TestMethod]
		public void ResolveRegularEscapeSequencesTest()
		{
			string escapedString, expected, actual;

			escapedString = @"\n";
			expected = "\n";
			actual = EscapeSequenceResolver.ResolveCommonEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);

			escapedString = @"\t";
			expected = "\t";
			actual = EscapeSequenceResolver.ResolveCommonEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);

			escapedString = @"\r";
			expected = "\r";
			actual = EscapeSequenceResolver.ResolveCommonEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);

			escapedString = @"\b";
			expected = "\b";
			actual = EscapeSequenceResolver.ResolveCommonEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);

			escapedString = @"\""";
			expected = "\"";
			actual = EscapeSequenceResolver.ResolveCommonEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);

			escapedString = @"\'";
			expected = "'";
			actual = EscapeSequenceResolver.ResolveCommonEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);

			escapedString = @"\\";
			expected = "\\";
			actual = EscapeSequenceResolver.ResolveCommonEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);

			escapedString = @"This\nstring\n\rcontains\t\tescapes";
			expected = "This\nstring\n\rcontains\t\tescapes";
			actual = EscapeSequenceResolver.ResolveCommonEscapeSequences(escapedString);
			Assert.AreEqual(expected, actual);
		}
	}
}
