﻿using System.Diagnostics;
using System.IO;
using System.Reflection;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using SharQL;

namespace SharQLTest
{
    /// <summary>
    /// Summary description for SyntaxTest
    /// </summary>
    [TestClass]
    public class SyntaxTests
    {
        private Parser parser;
        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        public bool ParseQueryFile(string prefix)
        {
            // Identify calling method name to deduce test query file
            StackFrame stackFrame = new StackFrame(1, true);
            MethodBase methodBase = stackFrame.GetMethod();
            string source;

            string testQueryFile = "syntax-" + methodBase.Name.Replace("__", "\\" + prefix + "-").Replace("_", "-") + ".rq";

            using (FileStream fs = new FileStream("..\\..\\..\\Tests\\" + testQueryFile, FileMode.Open))
            using (StreamReader reader = new StreamReader(fs))
            {
                source = reader.ReadToEnd();
            }

            try
            {
                parser = new Parser();
                bool result = parser.Parse(source);
                return result;
            }
            catch
            {
                return false;
            }
        }

        // Test Set 1

        [TestMethod]
        public void sparql1__basic_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__basic_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__basic_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__basic_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__basic_05() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__basic_06() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__bnodes_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__bnodes_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__bnodes_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__bnodes_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__bnodes_05() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__expr_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__expr_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__expr_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__expr_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__expr_05() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__forms_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__forms_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__limit_offset_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__limit_offset_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__limit_offset_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__limit_offset_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lists_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lists_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lists_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lists_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lists_05() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_05() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_06() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_07() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_08() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_09() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_10() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_11() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_12() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_13() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_14() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_15() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_16() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_17() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_18() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_19() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__lit_20() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__order_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__order_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__order_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__order_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__order_05() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__order_06() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__order_07() { Assert.IsTrue(ParseQueryFile("syntax")); }       
        [TestMethod]
        public void sparql1__pat_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__pat_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__pat_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__pat_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__qname_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__qname_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__qname_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__qname_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__qname_05() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__qname_06() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__qname_07() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__qname_08() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__struct_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__struct_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__struct_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__struct_05() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__struct_06() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__struct_07() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__struct_08() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__struct_09() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__struct_10() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__struct_11() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__struct_12() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__struct_13() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__struct_14() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__union_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql1__union_02() { Assert.IsTrue(ParseQueryFile("syntax")); }

        // Test Set 2

        [TestMethod]
        public void sparql2__bnode_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__bnode_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__bnode_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__dataset_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__dataset_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__dataset_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__dataset_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__esc_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__esc_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__esc_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__esc_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__esc_05() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__form_ask_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__form_construct01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__form_construct02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__form_construct03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__form_construct04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__form_construct06() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__form_describe01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__form_describe02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__form_select_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__form_select_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__function_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__function_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__function_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__function_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__general_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__general_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__general_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__general_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__general_05() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__general_06() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__general_07() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__general_08() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__general_09() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__general_10() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__general_11() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__general_12() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__general_13() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__general_14() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__graph_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__graph_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__graph_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__graph_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__graph_05() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__keywords_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__keywords_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__keywords_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__lists_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__lists_02() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__lists_03() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__lists_04() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql2__lists_05() { Assert.IsTrue(ParseQueryFile("syntax")); }

        // Test Set 3

        [TestMethod]
        public void sparql3__01() { Assert.IsTrue(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__02() { Assert.IsTrue(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__03() { Assert.IsTrue(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__04() { Assert.IsTrue(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__05() { Assert.IsTrue(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__06() { Assert.IsTrue(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__07() { Assert.IsTrue(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__08() { Assert.IsTrue(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_01() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_02() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_03() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_04() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_05() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_06() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_07() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_08() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_09() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_10() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_11() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_12() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_13() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_14() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_15() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_16() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_17() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_18() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_19() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_20() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_21() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_22() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_23() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_24() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_25() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_26() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_27() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_28() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_29() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_30() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_31() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_bnode_dot() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_bnodes_missing_pvalues_01() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_bnodes_missing_pvalues_02() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_empty_optional_01() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_empty_optional_02() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_filter_missing_parens() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_lone_list() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__bad_lone_node() { Assert.IsFalse(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql3__blabel_cross_filter() { Assert.IsTrue(ParseQueryFile("syn")); }

		//// Line 7, column 7: Blank node reused across basic graph patterns: _:who
		//[TestMethod]
		//public void sparql3__blabel_cross_graph_bad() { Assert.IsFalse(ParseQueryFile("syn")); }

		//// Line 9, column 7: Blank node reused across basic graph patterns: _:who
		//[TestMethod]
		//public void sparql3__blabel_cross_optional_bad() { Assert.IsFalse(ParseQueryFile("syn")); }

		//// Line 11, column 7: Blank node reused across basic graph patterns: _:who
		//[TestMethod]
		//public void sparql3__blabel_cross_union_bad() { Assert.IsFalse(ParseQueryFile("syn")); }

        // Test Set 4

        [TestMethod]
        public void sparql4__09() { Assert.IsTrue(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql4__10() { Assert.IsTrue(ParseQueryFile("syn")); }
        [TestMethod]
        public void sparql4__11() { Assert.IsTrue(ParseQueryFile("syn")); }

		//// Line 5, column 17: Blank node reused across basic graph patterns: _:a
		//[TestMethod]
		//public void sparql4__bad_34() { Assert.IsFalse(ParseQueryFile("syn")); }

		//// Line 5, column 19: Blank node reused across basic graph patterns: _:a
		//[TestMethod]
		//public void sparql4__bad_35() { Assert.IsFalse(ParseQueryFile("syn")); }

		//// Line 5, column 27: Blank node reused across basic graph patterns: _:a
		//[TestMethod]
		//public void sparql4__bad_36() { Assert.IsFalse(ParseQueryFile("syn")); }

		//// Line 5, column 19: Blank node reused across basic graph patterns: _:a
		//[TestMethod]
		//public void sparql4__bad_37() { Assert.IsFalse(ParseQueryFile("syn")); }

		//// Line 5, column 25: Blank node reused across basic graph patterns: _:a
		//[TestMethod]
		//public void sparql4__bad_38() { Assert.IsFalse(ParseQueryFile("syn")); }

		//// Line 8, column 37: Blank node reused across basic graph patterns: _:a
		//[TestMethod]
		//public void sparql4__bad_graph_breaks_BGP() { Assert.IsFalse(ParseQueryFile("syn")); }

		//// Line 8, column 37: Blank node reused across basic graph patterns: _:a
		//[TestMethod]
		//public void sparql4__bad_opt_breaks_BGP() { Assert.IsFalse(ParseQueryFile("syn")); }

		//// Line 10, column 50: Blank node reused across basic graph patterns: _:a
		//[TestMethod]
		//public void sparql4__bad_union_breaks_BGP() { Assert.IsFalse(ParseQueryFile("syn")); }

        [TestMethod]
        public void sparql4__leading_digits_in_prefixed_names() { Assert.IsTrue(ParseQueryFile("syn")); }

        // Test Set 5

        [TestMethod]
        public void sparql5__reduced_01() { Assert.IsTrue(ParseQueryFile("syntax")); }
        [TestMethod]
        public void sparql5__reduced_02() { Assert.IsTrue(ParseQueryFile("syntax")); }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        //[TestInitialize()]
        //public void TestInitialize() { }

        // Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void TestCleanup() { }
        //
        #endregion
    }
}
