﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SharQL.Ast;
using System.Windows.Forms;

namespace SharQLTestClient
{
	internal class TreeBuilderVisitor : SharQL.Ast.Visitor.IVisitor
	{
		/// <summary>Gets the map of tree nodes created.</summary>
		public Dictionary<INode, TreeNode> TreeNodes { get; private set; }

		/// <summary>Indexer required for visitor support.</summary>
		/// <param name="nodeType">Ignored</param>
		/// <returns>A delegate for translating AST node into 
		/// TreeNode object.</returns>
		public Action<INode> this[Type nodeType] { get { return constructTreeNode; } }

		/// <summary>Constructor</summary>
		public TreeBuilderVisitor()
		{
			TreeNodes = new Dictionary<INode, TreeNode>();
		}

		private void constructTreeNode(INode node)
		{
			NodeBase nb = node as NodeBase;
			if (nb != null)
			{
				// TreeNodes maps INode objects to TreeNode objects
				TreeNodes[node] = new TreeNode(nb.ToString().Split('.').Last());
				TreeNodes[node].Tag = node;
				foreach (INode child in nb.Children)
				{
					if (child != null && TreeNodes.ContainsKey(child))
					{
						TreeNodes[node].Nodes.Add(TreeNodes[child]);
					}
				}
			}
		}
	}
}
